/***************************************************************************
 *   Copyright (C) 2005 by Bjoern Erik Nilsen & Fredrik Berg Kjoelstad     *
 *   bjoern_erik_nilsen@hotmail.com & fredrikbk@hotmail.com                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef EXTERNALCHANGEMONITOR_H
#define EXTERNALCHANGEMONITOR_H

#include "src/config.h"
#include "animationmodel.h"

#include <fam.h>
#include <pthread.h>


/**
 * Class for listening after changes to the disc representation of the 
 * animationmodel.
 *
 * The class use FAM (File Alteration Monitor) register for changes
 * in the project directories.
 *
 * @author Bjoern Erik Nilsen & Fredrik Berg Kjoelstad
 */
class ExternalChangeMonitor
{
public:
	/**
	 * Initializes the ExternalChangeMonitor and registers the animationModel
	 * to notify of changes when the disk representation of the animation is 
	 * changed
	 * @param animationModel the animationModel to notify if the disc representation
	 * of the animation is changed.
	 */
	ExternalChangeMonitor(AnimationModel *animationModel);
	
	/**
	 * Register the current workdirectory for monitorng.
	 * @param workDirectory the workDirectory to listen for changes in.
	 */
	void changeWorkDirectory(const char* workDirectory);
	
	/**
	 * Creates a FAM connection to the project directories and starts a thread
	 * which polls the FAM connection for changes in the tmp directory.
	 *
	 * To register the working project directory for monitoring also call 
	 * changeWorkDirectory
	 */
	void startMonitoring();
	
	/**
	 * Stops the thread polling the FAM connection and tears the connection down.
	 */
	void stopMonitoring();
	
	/**
	 * Runs a loop indefinetly which polls the FAMConnection for changes
	 * in the project directory.
	 *
	 * NOTE: This function is primarly an internal function, but if you
	 *       still want to use it you should do so in a thread.
	 */
	void monitorFiles();
	
private:
	/** The animation to notify of changes */
	AnimationModel *animationModel;
	
	/** The thread which polls fam for changes in the project directories */
	pthread_t monitorThread;
	
	FAMConnection famConn;
	FAMRequest    famRequest;
	FAMEvent      famEvent;
	
	string workDirectory;
	string tmpDirectory;
};

#endif
